<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.1" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:msxsl="urn:schemas-microsoft-com:xslt" xmlns:inr="http://mycompany.com/mynamespace">
	<xsl:include href="../format.xsl"/>
	<xsl:param name="xslRootDirectory" select="inr:xslRootDirectory"/>
	<!-- Unique cross section surfaces -->
	<xsl:variable name="uniqueSurfaceName" select="//CrossSectionSurfaces/CrossSectionSurface[not (@name = preceding::CrossSectionSurface/@name)]/@name"/>
	<!-- Variable to hold number of points reported per line.  Change this value to suit requirements. -->
	<!-- You must also change the table data cells in the showPointData template below to match if you -->
	<!-- change this variable to be other than 5. -->
	<xsl:variable name="pointsPerLine" select="9"/>
	<!-- Cross Section Points Report Created By Will Rohman_2021-12-10-->
	<xsl:template match="/">
		<xsl:variable name="gridOut" select="inr:SetGridOut(number(InRoads/@outputGridScaleFactor))"/>
		<html>
			<head>
				<link rel="stylesheet" type="text/css" href="{$xslRootDirectory}/_Themes/engineer/theme.css"/>
				<!-- Title displayed in browser Title Bar -->
				<title lang="en">Cross Section Evaluation Report</title>
			</head>
			<body>
				<xsl:choose>
					<xsl:when test="$xslShowHelp = 'true'">
						<xsl:call-template name="StyleSheetHelp"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:for-each select="InRoads">
							<center>
								<!-- Report Title -->
								<h2 lang="en">Cross Section Evaluation Report</h2>
								<p lang="en">
                                    Report Created:&#xa0; <xsl:value-of select="inr:date()"/><br/>
									Time:&#xa0; <xsl:value-of select="inr:time()"/>
								</p>
							</center>
							<!-- Cross Section Set Data -->
							<xsl:for-each select="CrossSectionSet">
								<table class="margin" cellpadding="2" width="90%">
									<tbody>
										<tr>
											<th align="right" lang="en">Set Name:&#xa0; </th>
											<td align="left" colspan="2"><xsl:value-of select="@setName"/></td>
										</tr>
										<tr>
											<th align="right" lang="en">Alignment Name:&#xa0; </th>
											<td align="left" colspan="2"><xsl:value-of select="@alignmentName"/></td>
										</tr>
										<tr>
											<th align="right" lang="en" style="font-size: 80%">Input Grid Factor:&#xa0; </th>
											<td align="left" style="font-size: 80%"><xsl:value-of select="../@inputGridScaleFactor"/></td>
											<td align="right" lang="en" style="font-size: 80%">
												<strong>Note:&#xa0; </strong>All units in this report are in
                                                <xsl:if test="//@linearUnits = 'Imperial'">feet</xsl:if>
												<xsl:if test="//@linearUnits = 'Metric'">meters</xsl:if>
                                                unless specified otherwise.
                                            </td>
										</tr>
									</tbody>
								</table>
								<hr/>
								<!-- Cross Section Point Data -->
								<table width="100%">
									<xsl:for-each select="$uniqueSurfaceName">
										<tr>
											<th align="left" lang="en">Surface:&#xa0; </th>
											<td align="left" colspan="2"><xsl:value-of select="."/></td>
										</tr>
										<tr>
											<th class="underline" lang="en">Station</th>
											<xsl:call-template name="colHead">
												<xsl:with-param name="count" select="$pointsPerLine"/>
											</xsl:call-template>
										</tr>
										<xsl:for-each select="//CrossSectionSurface[@name = current()]">
											<tr>
												<td align="right" nowrap="nowrap">
													<br/><xsl:value-of select="inr:stationFormat(number(../../Station/@externalStation), $xslStationFormat,$xslStationPrecision, string(../../Station/@externalStationName))"/>
												</td>
												<xsl:for-each select="CrossSectionPoints">
													<!-- <xsl:for-each select="CrossSectionPoint[@type='ExistingCenterline']"> -->
													<xsl:for-each select="CrossSectionPoint[@offset = 0]">
														<xsl:call-template name="showPointData">
															<!-- <xsl:with-param name="list" select="CrossSectionPoint"/> -->
														</xsl:call-template>
													</xsl:for-each>
												</xsl:for-each>
											</tr>
										</xsl:for-each>
										<tr><td colspan="3">&#xa0;</td></tr>
									</xsl:for-each>
								</table>
							</xsl:for-each>
						</xsl:for-each>
					</xsl:otherwise>
				</xsl:choose>
			</body>
		</html>
	</xsl:template>
	<xsl:template name="colHead">
		<xsl:param name="count"/>
		<xsl:if test="$count != 0">
			<th class="underline" lang="en">Elevation</th>
			<th class="underline" lang="en">Distance</th>
			<th class="underline" lang="en">Cross_Slope</th>
			<xsl:call-template name="colHead">
				<xsl:with-param name="count" select="$count - 1"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	<xsl:template name="showPointData">
		<!-- Pnt1 -->
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:elevationFormat(number(preceding-sibling::CrossSectionPoint[4]/@elevation), $xslElevationPrecision)"/>
		</td>
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:distanceFormat(number(preceding-sibling::CrossSectionPoint[4]/@offset - preceding-sibling::CrossSectionPoint[3]/@offset), $xslDistancePrecision)"/>
		</td>
		<!-- Slope1-2 -->
		<td align="center" valign="bottom">
		<xsl:variable name="ElvDif1-2" select="preceding-sibling::CrossSectionPoint[4]/@elevation - preceding-sibling::CrossSectionPoint[3]/@elevation"/>
		<xsl:if test="$ElvDif1-2 &lt; 0">
		-</xsl:if><xsl:value-of select="format-number(($ElvDif1-2 div (preceding-sibling::CrossSectionPoint[4]/@offset - preceding-sibling::CrossSectionPoint[3]/@offset)), '0.00%;0.00%')"/>
		</td>
		<!-- Pnt2 -->
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:elevationFormat(number(preceding-sibling::CrossSectionPoint[3]/@elevation), $xslElevationPrecision)"/>
		</td>
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:distanceFormat(number(preceding-sibling::CrossSectionPoint[3]/@offset - preceding-sibling::CrossSectionPoint[2]/@offset), $xslDistancePrecision)"/>
		</td>
		<!-- Slope2-3 -->
		<td align="center" valign="bottom">
		<xsl:variable name="ElvDif2-3" select="preceding-sibling::CrossSectionPoint[3]/@elevation - preceding-sibling::CrossSectionPoint[2]/@elevation"/>
		<xsl:if test="$ElvDif2-3 &lt; 0">
		-</xsl:if><xsl:value-of select="format-number(($ElvDif2-3 div (preceding-sibling::CrossSectionPoint[3]/@offset - preceding-sibling::CrossSectionPoint[2]/@offset)), '0.00%;0.00%')"/>
		</td>
		<!-- Pnt3 -->
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:elevationFormat(number(preceding-sibling::CrossSectionPoint[2]/@elevation), $xslElevationPrecision)"/>
		</td>
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:distanceFormat(number(preceding-sibling::CrossSectionPoint[2]/@offset - preceding-sibling::CrossSectionPoint[1]/@offset), $xslDistancePrecision)"/>
		</td>
		<!-- Slope3-4 -->
		<td align="center" valign="bottom">
		<xsl:variable name="ElvDif3-4" select="preceding-sibling::CrossSectionPoint[2]/@elevation - preceding-sibling::CrossSectionPoint[1]/@elevation"/>
		<xsl:if test="$ElvDif3-4 &lt; 0">
		-</xsl:if><xsl:value-of select="format-number($ElvDif3-4 div (preceding-sibling::CrossSectionPoint[2]/@offset - preceding-sibling::CrossSectionPoint[1]/@offset), '0.00%;0.00%')"/>
		</td>
		<!-- Pnt4 -->
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:elevationFormat(number(preceding-sibling::CrossSectionPoint[1]/@elevation), $xslElevationPrecision)"/>
		</td>
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:distanceFormat(number(preceding-sibling::CrossSectionPoint[1]/@offset), $xslDistancePrecision)"/>
		</td>
		<!-- Slope4-5 -->
		<td align="center" valign="bottom">
		<xsl:variable name="ElvDif4-5" select="preceding-sibling::CrossSectionPoint[1]/@elevation - ./@elevation"/>
		<xsl:if test="$ElvDif4-5 &lt; 0">
		-</xsl:if><xsl:value-of select="format-number($ElvDif4-5 div preceding-sibling::CrossSectionPoint[1]/@offset, '0.00%;0.00%')"/>
		</td>
		<!-- Pnt5 -->
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:elevationFormat(number(./@elevation), $xslElevationPrecision)"/>
		</td>
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:distanceFormat(number(./@offset), $xslDistancePrecision)"/>
		</td>
		<!-- SlopeCL -->
		<td align="center" valign="bottom">
		CL
		</td>
		<!-- Pnt6 -->
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:elevationFormat(number(following-sibling::CrossSectionPoint[1]/@elevation), $xslElevationPrecision)"/>
		</td>
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:distanceFormat(number(following-sibling::CrossSectionPoint[1]/@offset), $xslDistancePrecision)"/>
		</td>
		<!-- Slope5-6 -->
		<td align="center" valign="bottom">
		<xsl:variable name="ElvDif6-5" select="following-sibling::CrossSectionPoint[1]/@elevation - ./@elevation"/>
		<xsl:if test="$ElvDif6-5 &lt; 0">
		-</xsl:if><xsl:value-of select="format-number($ElvDif6-5 div following-sibling::CrossSectionPoint[1]/@offset, '0.00%;0.00%')"/>
		</td>
		<!-- Pnt7 -->
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:elevationFormat(number(following-sibling::CrossSectionPoint[2]/@elevation), $xslElevationPrecision)"/>
		</td>
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:distanceFormat(number(following-sibling::CrossSectionPoint[2]/@offset - following-sibling::CrossSectionPoint[1]/@offset), $xslDistancePrecision)"/>
		</td>
		<!-- Slope6-7 -->
		<td align="center" valign="bottom">
		<xsl:variable name="ElvDif7-6" select="following-sibling::CrossSectionPoint[2]/@elevation - following-sibling::CrossSectionPoint[1]/@elevation"/>
		<xsl:if test="$ElvDif7-6 &lt; 0">
		-</xsl:if><xsl:value-of select="format-number(($ElvDif7-6 div (following-sibling::CrossSectionPoint[2]/@offset - following-sibling::CrossSectionPoint[1]/@offset)), '0.00%;0.00%')"/>
		</td>
		<!-- Pnt8 -->
		<td align="center" valign="bottom">								
		<xsl:value-of select="inr:elevationFormat(number(following-sibling::CrossSectionPoint[3]/@elevation), $xslElevationPrecision)"/>
		</td>
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:distanceFormat(number(following-sibling::CrossSectionPoint[3]/@offset - following-sibling::CrossSectionPoint[2]/@offset), $xslDistancePrecision)"/>
		</td>
		<!-- Slope7-8 -->
		<td align="center" valign="bottom">
		<xsl:variable name="ElvDif8-7" select="following-sibling::CrossSectionPoint[3]/@elevation - following-sibling::CrossSectionPoint[2]/@elevation"/>
		<xsl:if test="$ElvDif8-7 &lt; 0">
		-</xsl:if><xsl:value-of select="format-number(($ElvDif8-7 div (following-sibling::CrossSectionPoint[3]/@offset - following-sibling::CrossSectionPoint[2]/@offset)), '0.00%;0.00%')"/>
		</td>
		<!-- Pnt9 -->
		<td align="center" valign="bottom">								
		<xsl:value-of select="inr:elevationFormat(number(following-sibling::CrossSectionPoint[4]/@elevation), $xslElevationPrecision)"/>
		</td>
		<td align="center" valign="bottom">
		<xsl:value-of select="inr:distanceFormat(number(following-sibling::CrossSectionPoint[4]/@offset - following-sibling::CrossSectionPoint[3]/@offset), $xslDistancePrecision)"/>
		</td>
		<!-- Slope8-9 -->
		<td align="center" valign="bottom">
		<xsl:variable name="ElvDif9-8" select="following-sibling::CrossSectionPoint[4]/@elevation - following-sibling::CrossSectionPoint[3]/@elevation"/>
		<xsl:if test="$ElvDif9-8 &lt; 0">
		-</xsl:if><xsl:value-of select="format-number(($ElvDif9-8 div (following-sibling::CrossSectionPoint[4]/@offset - following-sibling::CrossSectionPoint[3]/@offset)), '0.00%;0.00%')"/>
		</td>
		<!-- <xsl:param name="list"/> -->
		<!-- <xsl:param name="rest" select="$list[position() &gt; $pointsPerLine]"/> -->
		<!-- <xsl:if test="$list[1]">
			<td align="right" valign="bottom">
				<xsl:value-of select="inr:elevationFormat(number($list[1]/@elevation), $xslElevationPrecision)"/>
			</td>
			<td align="right" valign="bottom">
				<xsl:value-of select="inr:distanceFormat(number($list[1]/@offset), $xslDistancePrecision)"/>
			</td>
			<td align="right" valign="bottom">
				<xsl:value-of select="format-number((($list[2]/@elevation - $list[1]/@elevation) div $list[1]/@offset), '0.0%;0.0%')"/>
			</td>
		</xsl:if>
		<xsl:if test="$list[2]">
			<td align="right" valign="bottom">
				<xsl:value-of select="inr:elevationFormat(number($list[2]/@elevation), $xslElevationPrecision)"/>
			</td>
			<td align="right" valign="bottom">
				<xsl:value-of select="inr:distanceFormat(number($list[2]/@offset), $xslDistancePrecision)"/>
			</td>
			<td align="right" valign="bottom">
               CL no Slope
            </td>
		</xsl:if>
		<xsl:if test="$list[3]">
			<td align="right" valign="bottom">
				<xsl:value-of select="inr:elevationFormat(number($list[3]/@elevation), $xslElevationPrecision)"/>
			</td>
			<td align="right" valign="bottom">
				<xsl:value-of select="inr:distanceFormat(number($list[3]/@offset), $xslDistancePrecision)"/>
			</td>
			<td align="right" valign="bottom">
				<xsl:value-of select="format-number((($list[2]/@elevation - $list[3]/@elevation) div $list[3]/@offset), '0.0%;0.0%')"/>
			</td>
		</xsl:if>
		<xsl:if test="$list[4]">
			<td align="right" valign="bottom">
				<xsl:value-of select="inr:elevationFormat(number($list[4]/@elevation), $xslElevationPrecision)"/>
			</td>
			<td align="right" valign="bottom">
				<xsl:value-of select="inr:distanceFormat(number($list[4]/@offset), $xslDistancePrecision)"/>
			</td>
		</xsl:if>
		<xsl:if test="$list[5]">
			<td align="right" valign="bottom">
				<xsl:value-of select="inr:elevationFormat(number($list[5]/@elevation), $xslElevationPrecision)"/>
			</td>
			<td align="right" valign="bottom">
				<xsl:value-of select="inr:distanceFormat(number($list[5]/@offset), $xslDistancePrecision)"/>
			</td>
		</xsl:if> -->
		<!-- Add or remove here if the number of points per line is different from 5. -->
		<!-- <xsl:if test="$rest">
			<tr>
				<td>&#xa0;</td>
				<xsl:call-template name="showPointData">
					<xsl:with-param name="list" select="$rest"/>
				</xsl:call-template>
			</tr>
		</xsl:if> -->
	</xsl:template>
	<xsl:template name="StyleSheetHelp">
		<div class="section1">
			<h4 lang="en">Notes</h4>
			<p class="normal1" lang="en">
                You must have created cross sections along your alignment and the cross section set must 
                have the surfaces and features upon which you wish to report displayed.
            </p>
			<p class="normal1" lang="en">
                You can create the XML data file from the <em>Evaluation &gt; Cross Section &gt; Cross 
                Section Report</em> command or from the <em>End-Area Volumes</em> leaf of the <em>
                Evaluation &gt; Cross Section &gt; Cross Sections</em> command by toggling on the <em>
                Create XML Report</em> option on the <em>General</em> leaf.
            </p>
			<p class="normal1" lang="en">
                This report displays five cross section points and their associated data per line before 
                wrapping to the next line.&#xa0; You can change the number of points per line by changing 
                the value in the <code>pointsPerLine</code> variable near the top of the style sheet and by 
                adding or removing columns in the <code>showPointData</code> template.
            </p>
			<p class="small" lang="en">
				<em>&#xa9; 2006 Bentley Systems, Inc</em>
			</p>
		</div>
	</xsl:template>
</xsl:stylesheet>
