<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.1" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:msxsl="urn:schemas-microsoft-com:xslt" xmlns:inr="http://mycompany.com/mynamespace">
	<xsl:include href="../format.xsl"/>
	<xsl:param name="xslRootDirectory" select="inr:xslRootDirectory"/>
	<!-- Unique cross section surfaces -->
	<xsl:variable name="uniqueSurfaceName" select="//CrossSectionSurfaces/CrossSectionSurface[not (@name = preceding::CrossSectionSurface/@name)]/@name"/>
	<!-- Variable to hold number of points reported per line.  Change this value to suit requirements. -->
	<!-- You must also change the table data cells in the showPointData template below to match if you -->
	<!-- change this variable to be other than 5. -->
	<xsl:variable name="pointsPerLine" select="9"/>
	<!-- Cross Section Points Report Created By Will Rohman_2021-12-10-->
	<xsl:output method="text" media-type="text/plain" encoding="iso-8859-1"/>
	<xsl:template match="/">
		<xsl:variable name="gridOut" select="inr:SetGridOut(number(InRoads/@outputGridScaleFactor))"/>
		<xsl:choose>
			<xsl:when test="$xslShowHelp = 'true'">
				<xsl:call-template name="StyleSheetHelp"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select="InRoads/CrossSectionSet">
					<xsl:text/>
					<xsl:text>Surface_Name: </xsl:text>
					<xsl:value-of select="//CrossSectionSurface[1]/@name"/>
					<xsl:text>&#xd;</xsl:text>
					<xsl:text>Alignment_Name: </xsl:text>
					<xsl:value-of select="@alignmentName"/>
					<xsl:text>&#xd;</xsl:text>
					<xsl:text>CrossSection_Set_Name: </xsl:text>
					<xsl:value-of select="@setName"/>
					<xsl:text>&#xd;</xsl:text>
					<xsl:text>&#xd;</xsl:text>
					<xsl:text>STA.; Elev.; Dist; Slope; Elev.; Dist; Slope; Elev.; Dist; Slope Elev.; Dist; Slope; Elev.; Dist; Slope; Elev.; Dist; Slope; Elev.; Dist; Slope; Elev.; Dist; Slope; Elev.; Dist; Slope&#xd;</xsl:text>
					<xsl:text>&#xd;</xsl:text>
					<xsl:for-each select="//CrossSectionSurface">
						<xsl:value-of select="inr:stationFormat(number(./ancestor::CrossSectionStation/Station/@externalStation), $xslStationFormat,$xslStationPrecision, string(Station/@externalStationName))"/>
						<xsl:text>; </xsl:text>
						<xsl:for-each select="CrossSectionPoints">
							<xsl:for-each select="CrossSectionPoint[@offset = 0]">
								<!-- Pnt1 --><xsl:value-of select="inr:elevationFormat(number(preceding-sibling::CrossSectionPoint[4]/@elevation), $xslElevationPrecision)"/><xsl:text>; </xsl:text><xsl:value-of select="inr:distanceFormat(number(preceding-sibling::CrossSectionPoint[4]/@offset - preceding-sibling::CrossSectionPoint[3]/@offset), $xslDistancePrecision)"/><xsl:text>; </xsl:text>
								<!-- Slope1-2 --><xsl:variable name="ElvDif1-2" select="preceding-sibling::CrossSectionPoint[4]/@elevation - preceding-sibling::CrossSectionPoint[3]/@elevation"/><xsl:if test="$ElvDif1-2 &lt; 0">-</xsl:if><xsl:value-of select="format-number(($ElvDif1-2 div (preceding-sibling::CrossSectionPoint[4]/@offset - preceding-sibling::CrossSectionPoint[3]/@offset)), '0.00%;0.00%')"/><xsl:text>; </xsl:text>
								<!-- Pnt2 --><xsl:value-of select="inr:elevationFormat(number(preceding-sibling::CrossSectionPoint[3]/@elevation), $xslElevationPrecision)"/><xsl:text>; </xsl:text><xsl:value-of select="inr:distanceFormat(number(preceding-sibling::CrossSectionPoint[3]/@offset - preceding-sibling::CrossSectionPoint[2]/@offset), $xslDistancePrecision)"/><xsl:text>; </xsl:text>
								<!-- Slope2-3 --><xsl:variable name="ElvDif2-3" select="preceding-sibling::CrossSectionPoint[3]/@elevation - preceding-sibling::CrossSectionPoint[2]/@elevation"/><xsl:if test="$ElvDif2-3 &lt; 0">-</xsl:if><xsl:value-of select="format-number(($ElvDif2-3 div (preceding-sibling::CrossSectionPoint[3]/@offset - preceding-sibling::CrossSectionPoint[2]/@offset)), '0.00%;0.00%')"/><xsl:text>; </xsl:text>
								<!-- Pnt3 --><xsl:value-of select="inr:elevationFormat(number(preceding-sibling::CrossSectionPoint[2]/@elevation), $xslElevationPrecision)"/><xsl:text>; </xsl:text><xsl:value-of select="inr:distanceFormat(number(preceding-sibling::CrossSectionPoint[2]/@offset - preceding-sibling::CrossSectionPoint[1]/@offset), $xslDistancePrecision)"/><xsl:text>; </xsl:text>
								<!-- Slope3-4 --><xsl:variable name="ElvDif3-4" select="preceding-sibling::CrossSectionPoint[2]/@elevation - preceding-sibling::CrossSectionPoint[1]/@elevation"/><xsl:if test="$ElvDif3-4 &lt; 0">-</xsl:if><xsl:value-of select="format-number($ElvDif3-4 div (preceding-sibling::CrossSectionPoint[2]/@offset - preceding-sibling::CrossSectionPoint[1]/@offset), '0.00%;0.00%')"/><xsl:text>; </xsl:text>
								<!-- Pnt4 --><xsl:value-of select="inr:elevationFormat(number(preceding-sibling::CrossSectionPoint[1]/@elevation), $xslElevationPrecision)"/><xsl:text>; </xsl:text><xsl:value-of select="inr:distanceFormat(number(preceding-sibling::CrossSectionPoint[1]/@offset), $xslDistancePrecision)"/><xsl:text>; </xsl:text>
								<!-- Slope4-5 --><xsl:variable name="ElvDif4-5" select="preceding-sibling::CrossSectionPoint[1]/@elevation - ./@elevation"/><xsl:if test="$ElvDif4-5 &lt; 0">-</xsl:if><xsl:value-of select="format-number($ElvDif4-5 div preceding-sibling::CrossSectionPoint[1]/@offset, '0.00%;0.00%')"/><xsl:text>; </xsl:text>
								<!-- Pnt5 --><xsl:value-of select="inr:elevationFormat(number(./@elevation), $xslElevationPrecision)"/><xsl:text>; </xsl:text><xsl:value-of select="inr:distanceFormat(number(./@offset), $xslDistancePrecision)"/><xsl:text>; </xsl:text>
								<!-- SlopeCL --><xsl:text>CL</xsl:text><xsl:text>; </xsl:text>
								<!-- Pnt6 --><xsl:value-of select="inr:elevationFormat(number(following-sibling::CrossSectionPoint[1]/@elevation), $xslElevationPrecision)"/><xsl:text>; </xsl:text><xsl:value-of select="inr:distanceFormat(number(following-sibling::CrossSectionPoint[1]/@offset), $xslDistancePrecision)"/><xsl:text>; </xsl:text>
								<!-- Slope5-6 --><xsl:variable name="ElvDif6-5" select="following-sibling::CrossSectionPoint[1]/@elevation - ./@elevation"/><xsl:if test="$ElvDif6-5 &lt; 0">-</xsl:if><xsl:value-of select="format-number($ElvDif6-5 div following-sibling::CrossSectionPoint[1]/@offset, '0.00%;0.00%')"/><xsl:text>; </xsl:text>
								<!-- Pnt7 --><xsl:value-of select="inr:elevationFormat(number(following-sibling::CrossSectionPoint[2]/@elevation), $xslElevationPrecision)"/><xsl:text>; </xsl:text><xsl:value-of select="inr:distanceFormat(number(following-sibling::CrossSectionPoint[2]/@offset - following-sibling::CrossSectionPoint[1]/@offset), $xslDistancePrecision)"/><xsl:text>; </xsl:text>
								<!-- Slope6-7 --><xsl:variable name="ElvDif7-6" select="following-sibling::CrossSectionPoint[2]/@elevation - following-sibling::CrossSectionPoint[1]/@elevation"/><xsl:if test="$ElvDif7-6 &lt; 0">-</xsl:if><xsl:value-of select="format-number(($ElvDif7-6 div (following-sibling::CrossSectionPoint[2]/@offset - following-sibling::CrossSectionPoint[1]/@offset)), '0.00%;0.00%')"/><xsl:text>; </xsl:text>
								<!-- Pnt8 --><xsl:value-of select="inr:elevationFormat(number(following-sibling::CrossSectionPoint[3]/@elevation), $xslElevationPrecision)"/><xsl:text>; </xsl:text><xsl:value-of select="inr:distanceFormat(number(following-sibling::CrossSectionPoint[3]/@offset - following-sibling::CrossSectionPoint[2]/@offset), $xslDistancePrecision)"/><xsl:text>; </xsl:text>
								<!-- Slope7-8 --><xsl:variable name="ElvDif8-7" select="following-sibling::CrossSectionPoint[3]/@elevation - following-sibling::CrossSectionPoint[2]/@elevation"/><xsl:if test="$ElvDif8-7 &lt; 0">-</xsl:if><xsl:value-of select="format-number(($ElvDif8-7 div (following-sibling::CrossSectionPoint[3]/@offset - following-sibling::CrossSectionPoint[2]/@offset)), '0.00%;0.00%')"/><xsl:text>; </xsl:text>
								<!-- Pnt9 --><xsl:value-of select="inr:elevationFormat(number(following-sibling::CrossSectionPoint[4]/@elevation), $xslElevationPrecision)"/><xsl:text>; </xsl:text><xsl:value-of select="inr:distanceFormat(number(following-sibling::CrossSectionPoint[4]/@offset - following-sibling::CrossSectionPoint[3]/@offset), $xslDistancePrecision)"/><xsl:text>; </xsl:text>
								<!-- Slope8-9 --><xsl:variable name="ElvDif9-8" select="following-sibling::CrossSectionPoint[4]/@elevation - following-sibling::CrossSectionPoint[3]/@elevation"/><xsl:if test="$ElvDif9-8 &lt; 0">-</xsl:if><xsl:value-of select="format-number(($ElvDif9-8 div (following-sibling::CrossSectionPoint[4]/@offset - following-sibling::CrossSectionPoint[3]/@offset)), '0.00%;0.00%')"/><xsl:text>; </xsl:text>
							</xsl:for-each>
						</xsl:for-each>
						<xsl:text>&#xd;</xsl:text>
					</xsl:for-each>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="StyleSheetHelp">
		<div class="section1">
			<h4 lang="en">Notes</h4>
			<p class="normal1" lang="en">
                You must have created cross sections along your alignment and the cross section set must 
                have the surfaces and features upon which you wish to report displayed.
            </p>
			<p class="normal1" lang="en">
                You can create the XML data file from the <em>Evaluation &gt; Cross Section &gt; Cross 
                Section Report</em> command or from the <em>End-Area Volumes</em> leaf of the <em>
                Evaluation &gt; Cross Section &gt; Cross Sections</em> command by toggling on the <em>
                Create XML Report</em> option on the <em>General</em> leaf.
            </p>
			<p class="normal1" lang="en">
                This report displays five cross section points and their associated data per line before 
                wrapping to the next line.&#xa0; You can change the number of points per line by changing 
                the value in the <code>pointsPerLine</code> variable near the top of the style sheet and by 
                adding or removing columns in the <code>showPointData</code> template.
            </p>
			<p class="small" lang="en">
				<em>&#xa9; 2006 Bentley Systems, Inc</em>
			</p>
		</div>
	</xsl:template>
</xsl:stylesheet>
