#pragma		Version		1:0:4
#define		CVersion	"1:0:4"

// -----------------------------------------------------------------------  
// Module:	celldmp.mc
// Description:	Dumps cells to a design file and a report to a text file
// Author:	G.Houck, (c)1996 systems hk
// Date:	02-Feb-96
// Version:	1.0.3
// Revisions:	Made sure 3d is supported; 1:0:3; GHouck; 06-Apr-1997
// 		Added Library Compress option; 1:0:4; GHouck; 09-Jul-1998
// -----------------------------------------------------------------------  
#include	<basetype.h>
#include	<cexpr.h>
#include	<cmdclass.h>
#include	<cmdlist.h>
#include	<ctype.h>
#include	<dbdefs.h>
#include	<dberrs.h>
#include	<global.h>
#include	<math.h>
#include	<mdl.h>
#include	<msdefs.h>
#include	<mselems.h>
#include	<msinputq.h>
#include	<mslstyle.h>
#include	<refernce.h>
#include	<rscdefs.h>
#include	<scanner.h>
#include	<stdarg.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<stdlib\stdarg.h>
#include	<stdlib\stdio.h>
#include	<string.h>
#include	<system.h>
#include	<tagdata.h>
#include	<tcb.h>
#include	<userfnc.h>
#include	<userpref.h>

#include	<mdllib.fdf>
#include	<mscell.fdf>
#include	<mscnv.fdf>
#include	<mscolor.fdf>
#include	<msdb.fdf>
#include	<msdialog.fdf>
#include	<mselemen.fdf>
#include	<mselmdsc.fdf>
#include	<msfile.fdf>
#include	<msinput.fdf>
#include	<mslocate.fdf>
#include	<msmisc.fdf>
#include	<msoutput.fdf>
#include	<msparse.fdf>
#include	<msrmatrx.fdf>
#include	<msrsrc.fdf>
#include	<msscan.fdf>
#include	<msscell.fdf>
#include	<msselect.fdf>
#include	<msstate.fdf>
#include	<mstmatrx.fdf>
#include	<msvar.fdf>
#include	<msvec.fdf>
#include	<mswindow.fdf>
#include	<rdbmslib.fdf>


// -----------------------------------------------------------------------  
// Local defines
// -----------------------------------------------------------------------  
#define		MAXCELLS	2000
#define		max(a,b)	((a) >= (b) ? (a) : (b))

// -----------------------------------------------------------------------  
// Global declarations
// -----------------------------------------------------------------------  
char		logFile[64];
int		nColumns;
int		orgLevel, orgColor, orgWeight, orgStyle;
int		txtLevel, txtColor, txtWeight, txtStyle;
int		nCells;
int		compressLib = TRUE;
double		uorsPerMstr;
double		deltaXY;
double		xOrigin, yOrigin, zOrigin;
double		scaleOverride;
FILE		*fLog;

typedef struct cellTag
{
  char	nam[6];
  short	dsc[9];
} cellData_;

cellData_	cellData[MAXCELLS+1];

// -----------------------------------------------------------------------  
// Function declarations
// -----------------------------------------------------------------------  
Private void	initRun ( void );
Private void	Usage ( void );
Private int	getCmdArgs ( int argc, char *argv[] );
Private int	strCMPNAM ( cellData_ *a, cellData_ *b );
Private int	dumpTheCells ( void );
Private void	quitApplication ( void );


// -----------------------------------------------------------------------  
// main entry point
// -----------------------------------------------------------------------  
Private int	main ( int argc, char *argv[] )
{
  int			status;
  int			i;

// set the defaults 

  initRun( );

// get the command line arguments 

  status = getCmdArgs( argc,argv );
  if( !status ) {
    Usage( );
    quitApplication( );
    return( SUCCESS );
  }
  else if( status < 0 ) {
    mdlUtil_beep( 1 );
    Usage( );
    return( SUCCESS );
  }
    
// open the output file 

  if( *logFile ) {
    fLog = fopen( logFile,"wt" );
    if( !fLog ) {
      mdlOutput_printf( MSG_ERROR,"Cannot open file: %s",logFile );
      quitApplication( );
      return( SUCCESS );
    }
  }

// output the log header 

  if( fLog ) {
    fprintf( fLog,"\nCellDmp -- Cell Dump Utility -- v%s (c)1996 systems hk\n",CVersion );
    fprintf( fLog,"\nDesign File:  %-s\n",tcb->dgnfilenm );
    fprintf( fLog,  "Cell Library: %-s\n",tcb->celfilenm );
    fprintf( fLog,  "[compress it?]%-s\n",(compressLib?"Yes":"No") );
    fprintf( fLog,  "Origin-X/Y/Z: %.2f/%.2f/%.2f\n",xOrigin,yOrigin,zOrigin );
    fprintf( fLog,  "Delta-X/Y:    %.2f\n",deltaXY );
    fprintf( fLog,  "# of Columns: %d\n\n",nColumns );
    fprintf( fLog,"\nSeq CellNm #Bytes Description................" );
    fprintf( fLog,"\n--- ------ ------ ---------------------------\n" );
  }

// if specified, compress the cell library 

  if( compressLib )
    mdlSystem_compressLibrary( );
  
// dump the cell to design and report 

  mdlOutput_printf( MSG_COMMAND,"Cell Dump Utility v%s",CVersion );
  
  nCells = dumpTheCells( );

// close up shop 

  if( *logFile && fLog )
    fclose( fLog );
    
  quitApplication( );

  mdlOutput_printf( MSG_PROMPT,"%d Cells Processed.",nCells );
  
  return( SUCCESS );  
}

// -----------------------------------------------------------------------  
// quitApplication -	terminate this application
// -----------------------------------------------------------------------  
Private void	quitApplication ( void )
{
  mdlDialog_cmdNumberQueue( FALSE,CMD_MDL_UNLOAD,mdlSystem_getCurrTaskID(),TRUE );
  return;
}


// -----------------------------------------------------------------------  
// dumpTheCells -	performs the actual Cell placement
// -----------------------------------------------------------------------  
Private int	dumpTheCells ( void )
{
  char			cellName[16];
  char			cellDescr[64];
  char			outText[128];
  int			i;
  int			nlist;
  int			status;
  int			scanWords;
  int			eTyp;
  int			knt;
  ULong			adrBuf[2];
  ULong			adrNxt;
  ULong			filePos;
  ULong			adrOK;
  Scanlist		scanList;
  Dpoint3d		org;
  Dpoint3d		snap;
  TagSpec		tagSpec;
  boolean		disp;
  TagValue		tagVal;
  ULong			targId;
  ULong			cellSize;
  Dpoint3d		off;
  Dpoint3d		linePts[2];
  Dpoint3d		cellShape[8];
  Dpoint3d		sca;
  Dpoint3d		cellOrg;
  Dpoint3d		minLim, maxLim;
  Dpoint3d		range;
  TextSizeParam		txtSizPrm;
  RotMatrix		rMat;
  MSElement		txtElem;
  MSElement		lineElem;
  MSElementDescr	*cellDP;
  MSElementDescr	*TelemDP;
  MSElementDescr	*tempDP;

  mdlScan_initScanlist( &scanList );
  mdlScan_noRangeCheck( &scanList );
  scanList.scantype 	= ONEELEM | ELEMTYPE | NESTCELL;
  scanList.extendedType =	FILEPOS;
  scanList.typmask[0]   =	TMSK0_CELL_LIB;

  mdlScan_initialize( CELL_LIB,&scanList );

  filePos = 0L;
  scanWords = sizeof( adrBuf) / sizeof( short );

// load up all of the cells from the attached library 

  nlist = 0;
  while( mdlScan_file(adrBuf,&scanWords,sizeof(adrBuf),&adrNxt) != 10 ) {
    if( nlist >= MAXCELLS ) {
      mdlOutput_printf( MSG_ERROR,"Too many cells (%d); skipping rest.",nlist );
      break;
    }			 
    cellDP = NULL;
    tempDP = NULL;
    if( !mdlElmdscr_read(&tempDP,adrBuf[0],CELL_LIB,NULL,NULL) ) {
      mdlOutput_error( "Error reading cell [1]" );
      goto LBLnextCell;
    }
    if( mdlCell_getElmDscr(&cellDP,NULL,adrBuf[0],NULL,NULL,NULL,NULL,0,0,NULL) != SUCCESS ) {
      mdlOutput_error( "Error reading cell [2]" );
      goto LBLnextCell;
    }
    if( mdlCell_extract(NULL,NULL,NULL,NULL,cellName,&cellDP->el) != SUCCESS ) {
      mdlOutput_error( "Error reading cell [3]" );
      goto LBLnextCell;
    }
    mdlOutput_printf( MSG_STATUS,"Processing #%d '%s'",(nlist+1),cellName );
    memcpy( cellData[nlist].dsc,tempDP->el.cell_lib_hdr.descrip,18 );
    strncpy( cellData[nlist].nam,cellName,6 );
    ++nlist;

LBLnextCell:

    if( cellDP )
      mdlElmdscr_freeAll( &cellDP );
    if( tempDP )
      mdlElmdscr_freeAll( &tempDP );
  }

// sort the cells 

  if( nlist > 1 )
    qsort( cellData,nlist,sizeof(cellData_),strCMPNAM );

// output each one to the design file and report 

  for( knt=0; knt<nlist; knt++ ) {
    strncpy( cellName,cellData[knt].nam,6 );
    cellName[6] = '\000';
    org.x = xOrigin * uorsPerMstr + (deltaXY * uorsPerMstr * (double)(knt % nColumns));
    org.y = yOrigin * uorsPerMstr + (deltaXY * uorsPerMstr * (double)(knt / nColumns));
    org.z = zOrigin * uorsPerMstr;

    cellDP = NULL;
    if( mdlCell_getElmDscr(&cellDP,NULL,0L,&org,NULL,NULL,NULL,0,0,cellName) != SUCCESS ) {
      mdlOutput_printf( MSG_ERROR,"Error getting '%s'.",cellName );
      goto LBLnextCell2;
    }
    if( mdlCell_extract(&cellOrg,cellShape,NULL,NULL,NULL,&cellDP->el) != SUCCESS ) {
      mdlOutput_printf( MSG_ERROR,"Error extracting '%s'.",cellName );
      goto LBLnextCell2;
    }
    
    cellSize = mdlElement_igdsSize( &cellDP->el,CELL_LIB ) * 2L;
    TelemDP = cellDP->h.firstElem;
    while( TelemDP ) {
      cellSize += mdlElement_igdsSize( &TelemDP->el,CELL_LIB ) * 2L;
      TelemDP = TelemDP->h.next;
    }

// determine the x/y range of the cell 

    minLim = cellShape[0];
    maxLim = cellShape[0];
    for( i=1; i<8; i++ ) {
      if( cellShape[i].x > maxLim.x )
        maxLim.x = cellShape[i].x;
      if( cellShape[i].x < minLim.x )
        minLim.x = cellShape[i].x;
      if( cellShape[i].y > maxLim.y )
        maxLim.y = cellShape[i].y;
      if( cellShape[i].y < minLim.y )
        minLim.y = cellShape[i].y;
    }
    range.x = maxLim.x - minLim.x;
    range.y = maxLim.y - minLim.y;
    if( range.x == 0.0 && range.y == 0.0 ) {
      if( fLog )
        fprintf( fLog,"%%-E-Invalid cell data: %s\n",cellName );
      goto LBLnextCell2;
    }
    if( scaleOverride )
      sca.x = sca.y = sca.z = scaleOverride;
    else
      sca.x = sca.y = sca.z = 0.60 * ((deltaXY * uorsPerMstr) / max(range.x,range.y));
    org.x -= ((minLim.x - cellOrg.x) * sca.x);
    org.y -= ((minLim.y - cellOrg.y) * sca.y);
    org.z = zOrigin * uorsPerMstr;
    mdlCell_placeCell( 0L,&org,&sca,NULL,NULL,0,FALSE,1,2,cellName );

// if user specified an Origin level, use it 

    if( orgLevel ) {
      linePts[0].x = linePts[1].x = org.x;
      linePts[0].y = org.y - (deltaXY * uorsPerMstr / 10.0);
      linePts[1].y = org.y + (deltaXY * uorsPerMstr / 10.0);
      linePts[0].z = linePts[1].z = zOrigin * uorsPerMstr;
      mdlLine_create( &lineElem,NULL,linePts );
      mdlElement_setProperties( &lineElem,&orgLevel,NULL,NULL,NULL,NULL,NULL,NULL,NULL );
      mdlElement_setSymbology( &lineElem,&orgColor,&orgWeight,&orgStyle );
      mdlElement_add( &lineElem );
      mdlElement_display( &lineElem,NORMALDRAW );

      linePts[0].y = linePts[1].y = org.y;
      linePts[0].x = org.x - (deltaXY * uorsPerMstr / 10.0);
      linePts[1].x = org.x + (deltaXY * uorsPerMstr / 10.0);
      linePts[0].z = linePts[1].z = zOrigin * uorsPerMstr;
      mdlLine_create( &lineElem,NULL,linePts );
      mdlElement_setProperties( &lineElem,&orgLevel,NULL,NULL,NULL,NULL,NULL,NULL,NULL );
      mdlElement_setSymbology( &lineElem,&orgColor,&orgWeight,&orgStyle );
      mdlElement_add( &lineElem );
      mdlElement_display( &lineElem,NORMALDRAW );
    }
    
    if( txtLevel ) {
      org.x = xOrigin * uorsPerMstr + (deltaXY * uorsPerMstr * (double)(knt % nColumns));
      org.y = yOrigin * uorsPerMstr + (deltaXY * uorsPerMstr * (double)(knt / nColumns));
      org.y -= (deltaXY * uorsPerMstr * 0.10);
      org.z = zOrigin * uorsPerMstr;
      txtSizPrm.size.width = 0.05 * deltaXY * uorsPerMstr;
      txtSizPrm.size.height = 0.05 * deltaXY * uorsPerMstr;
      sprintf( outText,"%s[%.3f]",cellName,sca.x );
      mdlText_create( &txtElem,NULL,outText,&org,&txtSizPrm,NULL,NULL,NULL );
      mdlElement_setProperties( &txtElem,&txtLevel,NULL,NULL,NULL,NULL,NULL,NULL,NULL );
      mdlElement_setSymbology( &txtElem,&txtColor,&txtWeight,&txtStyle );
      mdlElement_add( &txtElem );
      mdlElement_display( &txtElem,NORMALDRAW );
    }

    mdlCnv_fromR50ToAscii( 27,cellData[knt].dsc,cellDescr );
    if( fLog )
      fprintf( fLog,"%3d %-6.6s %6ld %-s\n",knt,cellData[knt].nam,cellSize,cellDescr );

LBLnextCell2:

    if( cellDP ) {
      mdlElmdscr_freeAll( &cellDP );
      cellDP = NULL;
    }
  }

// place the final title line 

  if( txtLevel ) {
    org.x = xOrigin * uorsPerMstr;
    org.y = yOrigin * uorsPerMstr - (deltaXY * uorsPerMstr * 0.50);
    org.z = zOrigin * uorsPerMstr;
    txtSizPrm.size.width = 0.10 * deltaXY * uorsPerMstr;
    txtSizPrm.size.height = 0.10 * deltaXY * uorsPerMstr;
    sprintf( outText,"Cell Library: %-s",tcb->celfilenm );
    mdlText_create( &txtElem,NULL,outText,&org,&txtSizPrm,NULL,NULL,NULL );
    mdlElement_setProperties( &txtElem,&txtLevel,NULL,NULL,NULL,NULL,NULL,NULL,NULL );
    mdlElement_setSymbology( &txtElem,&txtColor,&txtWeight,&txtStyle );
    mdlElement_add( &txtElem );
    mdlElement_display( &txtElem,NORMALDRAW );
  }

  return( nlist );
}

// -----------------------------------------------------------------------  
// strCMPNAM - compare function for sorting Cells on name
// -----------------------------------------------------------------------  
Private int	strCMPNAM ( cellData_ *a, cellData_ *b )
{
  return( strncmp(a->nam,b->nam,6) );
}


// -----------------------------------------------------------------------  
// getCmdArgs -	process the command-line options/parameters
// -----------------------------------------------------------------------  
Private int	getCmdArgs ( int argc, char *argv[] )
{
  int           iarg;
  char          *s0, *s1, *s2;

  for( iarg=1; iarg<argc; iarg++ ) {
    strupr( argv[iarg] );
    s0 = argv[iarg];
    s1 = (argv[iarg]+1);
    s2 = (argv[iarg]+2);
    if( *s0 == '-' || *s0 == '/' ) {
      switch( *s1 ) {
        
        case 'L':			// output file spec		
          strcpy( logFile,s2 );
          break;
        case 'C':			// Compress Cell library	
	  if( !*s2 || toupper(*s2) == 'Y' )
	    compressLib = TRUE;
	  else
	    compressLib = FALSE;
	  break;
        case 'D':			// delta-x/y spacing		
          deltaXY = atof( s2 );
          break;
        case 'X':			// X-Origin			
          xOrigin = atof( s2 );
          break;
        case 'Y':			// Y-Origin			
          yOrigin = atof( s2 );
          break;
        case 'Z':			// Z-Origin			
          zOrigin = atof( s2 );
          break;
        case 'N':			// # of columns			
          nColumns = atoi( s2 );
          break;
        case 'O':			// do origins level		
	  if( toupper(*s2) == 'L' )
	    orgLevel = atoi( (s2+1) );
	  else if( toupper(*s2) == 'C' )
	    orgColor = atoi( (s2+1) );
	  else if( toupper(*s2) == 'W' )
	    orgWeight = atoi( (s2+1) );
	  else if( toupper(*s2) == 'S' )
	    orgStyle = atoi( (s2+1) );
          break;
        case 'T':			// dump text labels level	
	  if( toupper(*s2) == 'L' )
	    txtLevel = atoi( (s2+1) );
	  else if( toupper(*s2) == 'C' )
	    txtColor = atoi( (s2+1) );
	  else if( toupper(*s2) == 'W' )
	    txtWeight = atoi( (s2+1) );
	  else if( toupper(*s2) == 'S' )
	    txtStyle = atoi( (s2+1) );
          break;
        case 'S':			// scale override value		
	  scaleOverride = atof( s2 );
	  break;

        case 'H':
        case '?':
          return( 0 );
	default:                        // ...something else...         
          return( -1 );
	  break;
      }
    }
  }

  return( 1 );
}


// -----------------------------------------------------------------------  
// Usage -	dump Help/Usage text to DOS window
// -----------------------------------------------------------------------  
Private void	Usage ( void )
{
  printf( "\nCellDmp -- Cell Dump Utility -- v1.10 (c)1996 systems hk\n" );
  printf( "\n" );
  printf( "\n -Lfile	Output file spec (none)" );
  printf( "\n -Cyn	Compress Cell Library (yes)" );
  printf( "\n -Dnn	Delta-x/y spacing (25)" );
  printf( "\n -Xnn	X-Origin (10000)" );
  printf( "\n -Ynn	Y-Origin (10000)" );
  printf( "\n -Znn	Z-Origin (0)" );
  printf( "\n -Nnn	# of column" );
  printf( "\n -Olcws	Origin-tick level,color,weight,style (63,0,0,1)" );
  printf( "\n -Tlcws	Text-label level,color,weight,style (62,0,0,0)" );
  printf( "\n -H/?	Dump this Help/Usage" );

  return;
}

// -----------------------------------------------------------------------  
// initRun -	initialize variables for the run
// -----------------------------------------------------------------------  
Private void	initRun ( void )
{
  scaleOverride	= 0.0;
  deltaXY	= 25.0;
  xOrigin	= 10000.0;
  yOrigin	= 10000.0;
  zOrigin	= 0.0;
  nColumns	= 10;
  orgLevel	= 63;
  orgColor	= 0;
  orgWeight	= 0;
  orgStyle	= 1;
  txtLevel	= 62 ;
  txtColor	= 0;
  txtWeight	= 0;
  txtStyle	= 0;
  *logFile	= '\000';
  fLog		= (FILE *)0;
  
  uorsPerMstr = tcb->subpermast * tcb->uorpersub;

  return;
}
